#include <stdio.h>
#include <stdlib.h>

  /* Ucitava n i pise sve particije broja n,
     tj. sve rastave broja n u sumu nepadajucih
     prirodnih brojeva.
     Na kraju ispisuje i broj particija.
     Efikasni rekurzivni algoritam zbrajanjem
       (broj poziva = broj particija).

     Koristi globalno dinamicki alocirano polje
     za pribrojnike u particiji.
     Ispisuje i cijeli "trag" poziva sa svim postavljenim
     pribrojnicima i nadjenim particijama.
  */

    /* Globalni brojac rekuzivnih poziva funkcije. */

unsigned int broj_poziva = 0u;

    /* Globalni pokazivac na dinamicko polje pribrojnika. */

int *pribr;

    /* Nadji sve particije broja suma u kojima je prvi (sljedeci)
       pribrojnik barem jednak prvi, a br_pribr je broj do tada
       postavljenih pribrojnika u "globalnoj" particiji polaznog broja.
       Za br_pribr > 0, indeks zadnjeg postavljenog pribrojnika u
       polju pribr je br_pribr - 1. */

int particije(int suma, int prvi, int br_pribr)
{
    int i, broj = 0;

        /* Povecaj globalni brojac poziva funkcije. */
    ++broj_poziva;

        /* Test suma >= prvi potreban je samo za prvi poziv izvana.
           Ako ocekujemo da je vanjski poziv korektan, onda je
           jednostavno modificirati gornju granicu petlje:
             i <= suma  ->  i <= suma - i
           tako da to vrijedi za sve daljnje pozive. */

        /* Ovdje ocekujemo da je prvi <= suma. Tada ima
           bar jedna particija, ona za i = suma.
           Ta particija je sigurno zavrsena, jer je preostala suma
           za rastav (nakon dodavanja tog pribrojnika) jednaka
           suma - i = 0. */
        /* Obrada te particije ide odmah, zbog traga. */

    ++br_pribr;

        /* Ispis traga poziva (pravim redom). */

    printf("  suma[%2d] = %2d, prvi[%2d] = %2d",
           br_pribr, suma, br_pribr, prvi);

        /* Obrada particije za i = suma, sigurno je br_pribr > 0. */

    ++broj;
    pribr[br_pribr - 1] = suma;
    printf("   ->");    // Oznaka za nadjenu particiju.
    printf("  %2d", pribr[0]);
    for (i = 1; i < br_pribr; ++i)
        printf(" + %2d", pribr[i]);
    printf("\n");

        /* Rekurzivni pozivi - dodavanje sljedeceg pribrojnika i,
           za i <= suma - i < suma.
           Uociti da je preostala suma - i sigurno pozitivna, tj.
           particija nije gotova (fale pribrojnici). Test na GORNJOJ
           granici i <= suma - i osigurava da je u nastavku rekurzije
           prvi <= suma, tj. da sigurno postoje takve particije
           u kojima je sljedeci pribrojnik barem i. */
        /* Petlja ide OBRATNIM redom zato da dobijemo leksikografski
           sortirane particije tragom unaprijed.
           Bivsi test na GORNJOJ granici i <= suma - i, znaci 2*i <= suma,
           tj. polazna najveca vrijednost za i je suma / 2. */

    for (i = suma / 2; i >= prvi; --i) {
        pribr[br_pribr - 1] = i;
        broj += particije( suma - i, i, br_pribr ) ;
    }

    return broj;
}

int main(void)
{
    int n;

    printf(" Upisi prirodni broj n: ");
    scanf("%d", &n);

        /* Alokacija polja za pribrojnike. */

    if ((pribr = (int*) malloc(n * sizeof(int))) == NULL)
    {
        printf("Alokacija nije uspjela.\n");
        exit(-1);
    }

    printf("\n Pravi trag poziva funkcije i generiranih particija");
    printf(" broja %d:\n\n", n);
    printf("\n Broj particija p(%d) = %d\n",
           n, particije(n, 1, 0) );

    printf("\n Broj poziva funkcije = %u\n",
           broj_poziva);

    free(pribr);

    return 0;
}
