#include <stdio.h>
#include <stdlib.h>
#include "ffun_lib.h"

    /* Program za sortiranje datoteke cijelih brojeva.
       Algoritam = Bubble_sort:
         sortiranje zamjenama susjeda, uz skracenu vanjsku petlju,
         sve dok ima zamjena u unutarnjoj petlji
       (v. bubble_2, Prog_1, 13_PROGS). */

    /* Standardna varijanta - koristi genericke funkcije na datoteci
       koje simuliraju odgovarajuce operacije na polju. */

/* ======================================================================== */

    /* Bubble_sort - Sortiranje vec otvorene datoteke f_obj
       koja sadrzi n_obj objekata, svaki velicine obj_size.
       Funkcija obj_cmp usporedjuje dva objekta zadana
       pokazivacima na njih, po ugledu na strcmp. */

    /* Genericka varijanta, simulacijom rada na polju. */

void g_bubble_sort(FILE *f_obj,
                   long int n_obj, size_t obj_size,
                   int (*obj_cmp) (const obj_type *, const obj_type *))
{
        /* "Indeksi" su tipa longint, kao kod ftell, fseek. */
    long int i, j;
    obj_type x_j, x_jp1;    /* x_j = x[j], x_jp1 = x[j + 1]. */
    int zamjena;


        /***** Bubble_sort. *****/

        /* Vanjska petlja za prolaze kroz datoteku.
           Indeks i je indeks zadnjeg elementa do kojeg
           treba ici u sljedecem prolazu kroz datoteku. */
    i = n_obj - 1L;
    do {
        zamjena = 0;
        for (j = 0L; j < i; ++j) {

                /* Ucitaj elemente x[j], x[j + 1]. */
            ucitaj_obj_i(f_obj, j, &x_j, obj_size);
            ucitaj_obj_i(f_obj, j + 1, &x_jp1, obj_size);

            if (obj_cmp(&x_j, &x_jp1) > 0) {

                    /* Zamjena x[j] i x[j + 1]. */
                zamjena_obj_ij(f_obj, j, j + 1, obj_size);
                zamjena = 1;
            }
        }

            /* Smanji i za sljedeci prolaz. */
        --i;
    } while (zamjena);


        /* Postavi se na pocetak datoteke (za svaki slucaj). */
    rewind(f_obj);

    return;
}

/* ======================================================================== */

    /* Glavni program. */

int main(int argc, char *argv[])
{
    FILE *f_obj;
    long int n_obj;

    if (argc != 2) {    /* Nema imena datoteke! */
        fprintf(stderr, "Uporaba: %s ime_dat\n",
                argv[0]);
        exit(1);
    }

        /* Otvaranje za binarno citanje i pisanje. */
    if ((f_obj = fopen(argv[1], "rb+")) == NULL) {
        fprintf(stderr, "Ne mogu pisati u: %s!\n",
                argv[1]);
        exit(2);
    }

        /* Priprema za sortiranje: nadji broj objekata u datoteci. */
    n_obj = broj_objekata(f_obj, obj_size);

        /* Ispis sadrzaja prije sortiranja. */
    printf("\n Sadrzaj datoteke %s --- prije sortiranja:\n", argv[1]);
    ispisi_dat(f_obj, obj_size);

        /* Pozicioniranje na pocetak, uz brisanje indikatora.
           Nije nuzno, jer sortiranje radi direktnim pristupom. */
    rewind(f_obj);


        /* Sortiranje datoteke. */
    g_bubble_sort(f_obj, n_obj, obj_size, obj_cmp);

        /* Ispis sadrzaja nakon sortiranja. */
    printf("\n Sadrzaj datoteke %s --- nakon sortiranja:\n", argv[1]);
    ispisi_dat(f_obj, obj_size);

        /* Provjera sortiranosti datoteke. */
    if (provjeri_sort_dat(f_obj, obj_size))
        printf("\n Datoteka je uzlazno sortirana.\n");
    else
        printf("\n Greska, datoteka NIJE uzlazno sortirana.\n");

        /* Zatvori datoteku. */
    fclose(f_obj);

    return 0;
}
