#include <stdio.h>
#include <stdlib.h>
#include "ffun_lib.h"

    /* Program za sortiranje datoteke cijelih brojeva.
       Algoritam = Selection_sort:
         sortiranje izborom ekstrema, uz dovodjenje najmanjeg elementa
         na pocetak nesredjenog dijela (skracuje se sprijeda)
       (v. sortmin1, Prog_1, 13_PROGS). */

    /* Standardna varijanta - koristi genericke funkcije na datoteci
       koje simuliraju odgovarajuce operacije na polju. */

/* ======================================================================== */

    /* Selection_sort - Sortiranje vec otvorene datoteke f_obj
       koja sadrzi n_obj objekata, svaki velicine obj_size.
       Funkcija obj_cmp usporedjuje dva objekta zadana
       pokazivacima na njih, po ugledu na strcmp. */

    /* Genericka varijanta, simulacijom rada na polju. */

void g_sel_sort_min(FILE *f_obj,
                    long int n_obj, size_t obj_size,
                    int (*obj_cmp) (const obj_type *, const obj_type *))
{
        /* "Indeksi" su tipa longint, kao kod ftell, fseek. */
    long int i, j, ind_min, i_max;
    obj_type x_min, x_j;    /* x_min = x[ind_min], x_j = x[j]. */


        /***** Selection_sort. *****/

    i_max = n_obj - 1L;
    for (i = 0L; i < i_max; ++i) {

            /* Ucitaj element x[i] u x_min. */
        ucitaj_obj_i(f_obj, i, &x_min, obj_size);
        ind_min = i;

        for (j = i + 1L; j < n_obj; ++j) {

                /* Ucitaj element x[j]. */
            ucitaj_obj_i(f_obj, j, &x_j, obj_size);

            if (obj_cmp(&x_j, &x_min) < 0) {
                ind_min = j;
                x_min = x_j;
            }
        }
        if (i != ind_min) {

                /* Zamjena x[i] i x[ind_min]. */
            zamjena_obj_ij(f_obj, i, ind_min, obj_size);
        }
    }

        /* Postavi se na pocetak datoteke (za svaki slucaj). */
    rewind(f_obj);

    return;
}

/* ======================================================================== */

    /* Glavni program. */

int main(int argc, char *argv[])
{
    FILE *f_obj;
    long int n_obj;

    if (argc != 2) {    /* Nema imena datoteke! */
        fprintf(stderr, "Uporaba: %s ime_dat\n",
                argv[0]);
        exit(1);
    }

        /* Otvaranje za binarno citanje i pisanje. */
    if ((f_obj = fopen(argv[1], "rb+")) == NULL) {
        fprintf(stderr, "Ne mogu pisati u: %s!\n",
                argv[1]);
        exit(2);
    }

        /* Priprema za sortiranje: nadji broj objekata u datoteci. */
    n_obj = broj_objekata(f_obj, obj_size);

        /* Ispis sadrzaja prije sortiranja. */
    printf("\n Sadrzaj datoteke %s --- prije sortiranja:\n", argv[1]);
    ispisi_dat(f_obj, obj_size);

        /* Pozicioniranje na pocetak, uz brisanje indikatora.
           Nije nuzno, jer sortiranje radi direktnim pristupom. */
    rewind(f_obj);


        /* Sortiranje datoteke. */
    g_sel_sort_min(f_obj, n_obj, obj_size, obj_cmp);

        /* Ispis sadrzaja nakon sortiranja. */
    printf("\n Sadrzaj datoteke %s --- nakon sortiranja:\n", argv[1]);
    ispisi_dat(f_obj, obj_size);

        /* Provjera sortiranosti datoteke. */
    if (provjeri_sort_dat(f_obj, obj_size))
        printf("\n Datoteka je uzlazno sortirana.\n");
    else
        printf("\n Greska, datoteka NIJE uzlazno sortirana.\n");

        /* Zatvori datoteku. */
    fclose(f_obj);

    return 0;
}
