#include <stdio.h>
#include <stdlib.h>
#include "ffun_lib.h"

    /* Program za sortiranje datoteke cijelih brojeva.
       Algoritam = QuickSort:
         kljucni element je prvi u danom dijelu polja [l, d]
         i njega dovodimo na pravo mjesto
       (v. qsort_1, Prog_2, 01_PROGS). */

    /* Standardna varijanta - koristi genericke funkcije na datoteci
       koje simuliraju odgovarajuce operacije na polju. */

/* ======================================================================== */

    /* QuickSort - Sortiranje vec otvorene datoteke f_obj
       koja sadrzi n_obj objekata, svaki velicine obj_size.
       Funkcija obj_cmp usporedjuje dva objekta zadana
       pokazivacima na njih, po ugledu na strcmp. */

    /* Genericka varijanta, simulacijom rada na polju. */

void g_quick_sort(FILE *f_obj,
                  long int l, long int d, size_t obj_size,
                  int (*obj_cmp) (const obj_type *, const obj_type *))
{
        /* "Indeksi" su tipa longint, kao kod ftell, fseek. */
    long int i, j;
    obj_type x_l, x_i, x_j;    /* x_l = x[l], x_i = x[i], x_j = x[j]. */


        /***** Quick_sort. *****/

    if (l < d) {

            /* Ucitaj element x[l] u x_l. */
        ucitaj_obj_i(f_obj, l, &x_l, obj_size);

        i = l + 1L;
        j = d;

            /* Prolaz mora i za i == j */
        while (i <= j) {

                /* Pomak i unaprijed, preskakanjem x[i] <= x[l]. */
            while (i <= d) {

                    /* Ucitaj element x[i] u x_i. */
                ucitaj_obj_i(f_obj, i, &x_i, obj_size);

                if (obj_cmp(&x_i, &x_l) > 0) break;
                ++i;
            }

                /* Pomak j unatrag, preskakanjem x[j] > x[l]. */
            while (1) {

                    /* Ucitaj element x[j] u x_j. */
                ucitaj_obj_i(f_obj, j, &x_j, obj_size);

                if (obj_cmp(&x_j, &x_l) <= 0) break;
                --j;
            }

            if (i < j)
                    /* Zamjena x[i] i x[j]. */
                zamjena_obj_ij(f_obj, i, j, obj_size);
        }

        if (l < j)
                /* Zamjena x[j] i x[l]. */
            zamjena_obj_ij(f_obj, j, l, obj_size);

        g_quick_sort(f_obj, l, j - 1L, obj_size, obj_cmp);
        g_quick_sort(f_obj, j + 1L, d, obj_size, obj_cmp);
    }

        /* Postavi se na pocetak datoteke (za svaki slucaj). */
    rewind(f_obj);

    return;
}

/* ======================================================================== */

    /* Glavni program. */

int main(int argc, char *argv[])
{
    FILE *f_obj;
    long int n_obj;

    if (argc != 2) {    /* Nema imena datoteke! */
        fprintf(stderr, "Uporaba: %s ime_dat\n",
                argv[0]);
        exit(1);
    }

        /* Otvaranje za binarno citanje i pisanje. */
    if ((f_obj = fopen(argv[1], "rb+")) == NULL) {
        fprintf(stderr, "Ne mogu pisati u: %s!\n",
                argv[1]);
        exit(2);
    }

        /* Priprema za sortiranje: nadji broj objekata u datoteci. */
    n_obj = broj_objekata(f_obj, obj_size);

        /* Ispis sadrzaja prije sortiranja. */
    printf("\n Sadrzaj datoteke %s --- prije sortiranja:\n", argv[1]);
    ispisi_dat(f_obj, obj_size);

        /* Pozicioniranje na pocetak, uz brisanje indikatora.
           Nije nuzno, jer sortiranje radi direktnim pristupom. */
    rewind(f_obj);


        /* Sortiranje datoteke. */
    g_quick_sort(f_obj, 0, n_obj - 1, obj_size, obj_cmp);

        /* Ispis sadrzaja nakon sortiranja. */
    printf("\n Sadrzaj datoteke %s --- nakon sortiranja:\n", argv[1]);
    ispisi_dat(f_obj, obj_size);

        /* Provjera sortiranosti datoteke. */
    if (provjeri_sort_dat(f_obj, obj_size))
        printf("\n Datoteka je uzlazno sortirana.\n");
    else
        printf("\n Greska, datoteka NIJE uzlazno sortirana.\n");

        /* Zatvori datoteku. */
    fclose(f_obj);

    return 0;
}
