#include <stdio.h>
#include <stdlib.h>
#include "ffun_lib.h"

    /* Program za sortiranje datoteke cijelih brojeva.
       Algoritam = Selection_sort:
         sortiranje izborom ekstrema, uz dovodjenje najveceg elementa
         na kraj nesredjenog dijela (skracuje se straga)
       (v. sortmax1, Prog_1, 13_PROGS ili max_sort, Prog_2, 11_PROGS). */

    /* Ubrzana varijanta - koristi:
       - sekvencijalno citanje unaprijed
       - pamti pozicije i vrijednosti kod zamjena objekata. */

/* ======================================================================== */

    /* Selection_sort - Sortiranje vec otvorene datoteke f_obj
       koja sadrzi n_obj objekata, svaki velicine obj_size.
       Funkcija obj_cmp usporedjuje dva objekta zadana
       pokazivacima na njih, po ugledu na strcmp. */

    /* Ubrzana varijanta. */

void f_sel_sort_max(FILE *f_obj,
                    long int n_obj, size_t obj_size,
                    int (*obj_cmp) (const obj_type *, const obj_type *))
{
        /* "Indeksi" su tipa longint, kao kod ftell, fseek. */
    long int i, j, i_max, j_max;
    long int pos_max, pos_j;
    obj_type x_max, x_j;    /* x_max = x[ind_max], x_j = x[j]. */


        /***** Selection_sort. *****/

        /* Napomena: pozicija na koju treba spremiti najveci element
           je pozicija zadnjeg ucitanog x[j] u petlji po j. */

    i_max = n_obj - 1L;
    for (i = 0L; i < i_max; ++i) {

        rewind(f_obj);

            /* Ucitaj element x[0] u x_max kao trenutno najveci
               i zapamti mu poziciju. */
        ucitaj_obj(f_obj, &x_max, obj_size);
        pos_max = 0L;

            /* Napomena: Ako ne zelimo koristiti ftell u petlji ispod,
               onda ovdje inicijaliziramo pos_j = 0L; */

        j_max = i_max - i;
        for (j = 1L; j <= j_max; ++j) {

                /* Zapamti poziciju za x[j], prije citanja. */
                /* Ako ne zelimo koristiti ftell, onda ovdje treba staviti
                   pos_j += obj_size; */
            pos_j = ftell(f_obj);

                /* Ucitaj element x[j]. */
            ucitaj_obj(f_obj, &x_j, obj_size);

            if (obj_cmp(&x_j, &x_max) > 0) {
                pos_max = pos_j;
                x_max = x_j;
            }
        }

            /* Pozicija zadnjeg ucitanog x[j] je upravo pozicija
               na koju treba spremiti najveci element. */
        if (pos_j != pos_max) {

                /* Zamjena elemenata na pozicijama pos_j i pos_max. */
            postavi_fpos(f_obj, pos_j, SEEK_SET);
            napisi_obj(f_obj, &x_max, obj_size);
            postavi_fpos(f_obj, pos_max, SEEK_SET);
            napisi_obj(f_obj, &x_j, obj_size);

                /* Isprazni spremnik u datoteku nakon pisanja,
                   pri prijelazu s pisanja na citanje. */
            isprazni_spremnik(f_obj);
        }
    }

        /* Postavi se na pocetak datoteke (za svaki slucaj). */
    rewind(f_obj);

    return;
}

/* ======================================================================== */

    /* Glavni program. */

int main(int argc, char *argv[])
{
    FILE *f_obj;
    long int n_obj;

    if (argc != 2) {    /* Nema imena datoteke! */
        fprintf(stderr, "Uporaba: %s ime_dat\n",
                argv[0]);
        exit(1);
    }

        /* Otvaranje za binarno citanje i pisanje. */
    if ((f_obj = fopen(argv[1], "rb+")) == NULL) {
        fprintf(stderr, "Ne mogu pisati u: %s!\n",
                argv[1]);
        exit(2);
    }

        /* Priprema za sortiranje: nadji broj objekata u datoteci. */
    n_obj = broj_objekata(f_obj, obj_size);

        /* Ispis sadrzaja prije sortiranja. */
    printf("\n Sadrzaj datoteke %s --- prije sortiranja:\n", argv[1]);
    ispisi_dat(f_obj, obj_size);

        /* Pozicioniranje na pocetak, uz brisanje indikatora.
           Nije nuzno, jer sortiranje radi direktnim pristupom. */
    rewind(f_obj);


        /* Sortiranje datoteke. */
    f_sel_sort_max(f_obj, n_obj, obj_size, obj_cmp);

        /* Ispis sadrzaja nakon sortiranja. */
    printf("\n Sadrzaj datoteke %s --- nakon sortiranja:\n", argv[1]);
    ispisi_dat(f_obj, obj_size);

        /* Provjera sortiranosti datoteke. */
    if (provjeri_sort_dat(f_obj, obj_size))
        printf("\n Datoteka je uzlazno sortirana.\n");
    else
        printf("\n Greska, datoteka NIJE uzlazno sortirana.\n");

        /* Zatvori datoteku. */
    fclose(f_obj);

    return 0;
}
