#include <stdio.h>
#include <stdlib.h>
#include "ffun_lib.h"

    /* Program za sortiranje datoteke cijelih brojeva.
       Algoritam = Selection_sort:
         sortiranje izborom ekstrema, uz dovodjenje najmanjeg elementa
         na pocetak nesredjenog dijela (skracuje se sprijeda)
       (v. sortmin1, Prog_1, 13_PROGS). */

    /* Ubrzana varijanta - koristi:
       - sekvencijalno citanje unaprijed
       - pamti pozicije i vrijednosti kod zamjena objekata. */

/* ======================================================================== */

    /* Selection_sort - Sortiranje vec otvorene datoteke f_obj
       koja sadrzi n_obj objekata, svaki velicine obj_size.
       Funkcija obj_cmp usporedjuje dva objekta zadana
       pokazivacima na njih, po ugledu na strcmp. */

    /* Ubrzana varijanta. */

void f_sel_sort_min(FILE *f_obj,
                    long int n_obj, size_t obj_size,
                    int (*obj_cmp) (const obj_type *, const obj_type *))
{
        /* "Indeksi" su tipa longint, kao kod ftell, fseek. */
    long int i, j, i_max;
    long int pos_i, pos_min, pos_j;
    obj_type x_i, x_min, x_j;    /* x_i = x[i], x_min = x[ind_min],
                                    x_j = x[j]. */


        /***** Selection_sort. *****/

        /* Pozicija od koje pocinje citanje u petlji po j, ujedno
           i pozicija na koju treba spremiti najmanji element. */
    pos_i = 0L ;

    i_max = n_obj - 1L;
    for (i = 0L; i < i_max; ++i) {

        postavi_fpos(f_obj, pos_i, SEEK_SET);

            /* Ucitaj element x[i] u x_i. */
        ucitaj_obj(f_obj, &x_i, obj_size);

            /* Zapamti poziciju i vrijednost trenutno najmanjeg. */
        pos_min = pos_i;
        x_min = x_i;

            /* Napomena: Ako ne zelimo koristiti ftell u petlji ispod,
               onda ovdje inicijaliziramo pos_j = pos_i; */

        for (j = i + 1L; j < n_obj; ++j) {

                /* Zapamti poziciju za x[j], prije citanja. */
                /* Ako ne zelimo koristiti ftell, onda ovdje treba staviti
                   pos_j += obj_size; */
            pos_j = ftell(f_obj);

                /* Ucitaj element x[j]. */
            ucitaj_obj(f_obj, &x_j, obj_size);

            if (obj_cmp(&x_j, &x_min) < 0) {
                pos_min = pos_j;
                x_min = x_j;
            }
        }
        if (pos_i != pos_min) {

                /* Zamjena elemenata na pozicijama pos_i i pos_min. */
            postavi_fpos(f_obj, pos_i, SEEK_SET);
            napisi_obj(f_obj, &x_min, obj_size);
            postavi_fpos(f_obj, pos_min, SEEK_SET);
            napisi_obj(f_obj, &x_i, obj_size);

                /* Isprazni spremnik u datoteku nakon pisanja,
                   pri prijelazu s pisanja na citanje. */
            isprazni_spremnik(f_obj);
        }

            /* Pomakni poziciju najmanjeg elementa za sljedeci prolaz. */
        pos_i += obj_size;
    }

        /* Postavi se na pocetak datoteke (za svaki slucaj). */
    rewind(f_obj);

    return;
}

/* ======================================================================== */

    /* Glavni program. */

int main(int argc, char *argv[])
{
    FILE *f_obj;
    long int n_obj;

    if (argc != 2) {    /* Nema imena datoteke! */
        fprintf(stderr, "Uporaba: %s ime_dat\n",
                argv[0]);
        exit(1);
    }

        /* Otvaranje za binarno citanje i pisanje. */
    if ((f_obj = fopen(argv[1], "rb+")) == NULL) {
        fprintf(stderr, "Ne mogu pisati u: %s!\n",
                argv[1]);
        exit(2);
    }

        /* Priprema za sortiranje: nadji broj objekata u datoteci. */
    n_obj = broj_objekata(f_obj, obj_size);

        /* Ispis sadrzaja prije sortiranja. */
    printf("\n Sadrzaj datoteke %s --- prije sortiranja:\n", argv[1]);
    ispisi_dat(f_obj, obj_size);

        /* Pozicioniranje na pocetak, uz brisanje indikatora.
           Nije nuzno, jer sortiranje radi direktnim pristupom. */
    rewind(f_obj);


        /* Sortiranje datoteke. */
    f_sel_sort_min(f_obj, n_obj, obj_size, obj_cmp);

        /* Ispis sadrzaja nakon sortiranja. */
    printf("\n Sadrzaj datoteke %s --- nakon sortiranja:\n", argv[1]);
    ispisi_dat(f_obj, obj_size);

        /* Provjera sortiranosti datoteke. */
    if (provjeri_sort_dat(f_obj, obj_size))
        printf("\n Datoteka je uzlazno sortirana.\n");
    else
        printf("\n Greska, datoteka NIJE uzlazno sortirana.\n");

        /* Zatvori datoteku. */
    fclose(f_obj);

    return 0;
}
