#include <stdio.h>
#include <stdlib.h>
#include "ffun_lib.h"

    /* Program za sortiranje datoteke cijelih brojeva.
       Algoritam = QuickSort (Niklaus Wirth, A+DS=P):
         kljucni element je srednji u danom dijelu polja [l, d]
         i njega dovodimo na pravo mjesto
       (v. qsort_w, Prog_2, 01_PROGS). */

    /* Ubrzana varijanta - koristi:
       - sekvencijalno citanje unaprijed i pomake unatrag
       - pamti pozicije i vrijednosti kod zamjena objekata. */

/* ======================================================================== */

    /* QuickSort - Sortiranje vec otvorene datoteke f_obj
       koja sadrzi n_obj objekata, svaki velicine obj_size.
       Funkcija obj_cmp usporedjuje dva objekta zadana
       pokazivacima na njih, po ugledu na strcmp. */

    /* Ubrzana varijanta. */

void f_quick_sort_w(FILE *f_obj,
                    long int l, long int d, size_t obj_size,
                    int (*obj_cmp) (const obj_type *, const obj_type *))
{
        /* "Indeksi" su tipa longint, kao kod ftell, fseek. */
    long int i, j;
    long int pos_s, pos_i, pos_j;
    obj_type x_s, x_i, x_j;    /* x_s = x[s], x_i = x[i], x_j = x[j]. */

        /* Pomak za dva objekta natrag kod citanja unatrag. */
    long int dva_natrag = -2L * obj_size;


        /***** Quick_sort. *****/

        /* Test l >= d treba samo u prvom pozivu (izvana), kao zastita
           od prazne datoteke (d == -1L). Ako izvana osiguramo d >= 0,
           ovaj test se moze izbaciti, jer ostatak radi uvijek, ukljucivo
           i sve rekurzivne pozive (njima ovaj test ne treba, jer se test
           duljine radi prije rekurzivnih poziva). */
    if (l >= d) return;

        /* Postavi poziciju srednjeg elementa x[s], s = (l + d)/2. */
    pos_s = ((l + d) / 2L) * obj_size;
    postavi_fpos(f_obj, pos_s, SEEK_SET);

        /* Ucitaj element x[s] u x_s. */
    ucitaj_obj(f_obj, &x_s, obj_size);

    i = l;
    j = d;

        /* Pozicije elemenata x[i] = x[l] i x[j] = x[d]. */
    pos_i = l * obj_size;
    pos_j = d * obj_size;

        /* Petlja do-while (i <= j) za particioniranje polja.
           Prolaz mora i za i == j. */
    do {

            /***** Realizacija petlje  while (x[i] < x[s]) ++i; *****/

            /* Postavi poziciju za sljedeci x[i]. */
        postavi_fpos(f_obj, pos_i, SEEK_SET);

            /* Pomak i unaprijed, preskakanjem x[i] < x_s. */
        while (1) {

                /* Ucitaj element x[i] u x_i. */
            ucitaj_obj(f_obj, &x_i, obj_size);

            if (obj_cmp(&x_i, &x_s) >= 0) break;
            ++i;
        }
            /* Zapamti poziciju zadnjeg x[i]. */
        pos_i = ftell(f_obj) - obj_size;


            /***** Realizacija petlje  while (x[j] > x_s) --j; *****/

            /* Postavi poziciju za sljedeci x[j]. */
        postavi_fpos(f_obj, pos_j, SEEK_SET);

            /* Pomak j unatrag, preskakanjem x[j] > x[s]. */
        while (1) {

                /* Ucitaj element x[j] u x_j. */
            ucitaj_obj(f_obj, &x_j, obj_size);

            if (obj_cmp(&x_j, &x_s) <= 0) break;
            --j;

                /* Pomak unatrag za dva mjesta, za sljedeci x[j]. */
            postavi_fpos(f_obj, dva_natrag, SEEK_CUR);
        }
            /* Zapamti poziciju zadnjeg x[j]. */
        pos_j = ftell(f_obj) - obj_size;


            /* Ako je i <= j, zamijeni elemente x[i] i x[j], s tim da
               zamjena ne treba za i == j. Zatim, preskoci te elemente,
               pomakom i unaprijed, j unatrag. Pripadni kod je ispod. */
/*****
        if (i <= j) {
            swap(&x[i], &x[j]);
            ++i;
            --j;
        }
*****/
        if (i <= j) {
            if (i < j) {
                    /* Zamjena x[i] i x[j], na pozicijama pos_i, pos_j. */
                postavi_fpos(f_obj, pos_j, SEEK_SET);
                napisi_obj(f_obj, &x_i, obj_size);
                postavi_fpos(f_obj, pos_i, SEEK_SET);
                napisi_obj(f_obj, &x_j, obj_size);

                    /* Isprazni spremnik u datoteku nakon pisanja,
                       pri prijelazu s pisanja na citanje. */
                isprazni_spremnik(f_obj);
            }
                /* Pomakni i unaprijed, j unatrag za jedno mjesto.
                   Isto napravi s pozicijama elemenata pos_i, pos_j. */
            ++i;
            --j;

            pos_i += obj_size;
            pos_j -= obj_size;
        }

    } while (i <= j);

        /* Test duljine prije rekurzivnih poziva. */
    if (l < j) f_quick_sort_w(f_obj, l, j, obj_size, obj_cmp);
    if (i < d) f_quick_sort_w(f_obj, i, d, obj_size, obj_cmp);

        /* Postavi se na pocetak datoteke (za svaki slucaj). */
/*         Ovo nema smisla u rekurzivnoj funkciji!
    rewind(f_obj);
 */

    return;
}

/* ======================================================================== */

    /* Glavni program. */

int main(int argc, char *argv[])
{
    FILE *f_obj;
    long int n_obj;

    if (argc != 2) {    /* Nema imena datoteke! */
        fprintf(stderr, "Uporaba: %s ime_dat\n",
                argv[0]);
        exit(1);
    }

        /* Otvaranje za binarno citanje i pisanje. */
    if ((f_obj = fopen(argv[1], "rb+")) == NULL) {
        fprintf(stderr, "Ne mogu pisati u: %s!\n",
                argv[1]);
        exit(2);
    }

        /* Priprema za sortiranje: nadji broj objekata u datoteci. */
    n_obj = broj_objekata(f_obj, obj_size);

        /* Ispis sadrzaja prije sortiranja. */
    printf("\n Sadrzaj datoteke %s --- prije sortiranja:\n", argv[1]);
    ispisi_dat(f_obj, obj_size);

        /* Pozicioniranje na pocetak, uz brisanje indikatora.
           Nije nuzno, jer sortiranje radi direktnim pristupom. */
    rewind(f_obj);


        /* Sortiranje datoteke. */
    f_quick_sort_w(f_obj, 0, n_obj - 1, obj_size, obj_cmp);

        /* Ispis sadrzaja nakon sortiranja. */
    printf("\n Sadrzaj datoteke %s --- nakon sortiranja:\n", argv[1]);
    ispisi_dat(f_obj, obj_size);

        /* Provjera sortiranosti datoteke. */
    if (provjeri_sort_dat(f_obj, obj_size))
        printf("\n Datoteka je uzlazno sortirana.\n");
    else
        printf("\n Greska, datoteka NIJE uzlazno sortirana.\n");

        /* Zatvori datoteku. */
    fclose(f_obj);

    return 0;
}
